using System;
using System.Collections.Generic;
using System.IO;
using System.Security.Cryptography;
using System.Text;
using Utilities;

namespace QualcommLibrary
{
    public class ChainedExecutableVerification
    {
        public static byte[] DecryptFileSignature(string imagePath)
        {
            byte[] imageBytes = File.ReadAllBytes(imagePath);
            List<byte[]> certificates = ChainedExecutableHelper.ExtractCertificates(imageBytes);
            if (certificates.Count > 0)
            {
                byte[] certificateBytes = certificates[0];
                byte[] signatureBytes = ChainedExecutableHelper.ExtractSignature(imageBytes);
                RSAParameters rsaParameters = CertificateHelper.GetRSAParameters(certificateBytes);
                byte[] decodedHash = RSAHelper.DecryptSignature(signatureBytes, rsaParameters);
                return decodedHash;
            }
            else
            {
                throw new Exception("According to the header, the file does not contain a certificate");
            }
        }

        public static byte[] DecryptCertificateSignature(string imagePath, int certficateIndex)
        {
            byte[] imageBytes = File.ReadAllBytes(imagePath);
            List<byte[]> certificates = ChainedExecutableHelper.ExtractCertificates(imageBytes);
            byte[] parentCertificateBytes = certificates[certficateIndex + 1];
            byte[] certificateBytes = certificates[certficateIndex];
            RSAParameters rsaParameters = CertificateHelper.GetRSAParameters(parentCertificateBytes);
            byte[] certificateSignature = ByteReader.ReadBytes(certificateBytes, certificateBytes.Length - 256, 256);
            byte[] decodedHash = RSAHelper.DecryptSignature(certificateSignature, rsaParameters);
            return decodedHash;
        }
    }
}
